@@ -10,11 +10,13 @@ if os.path.isdir(DIR):
     sys.path.insert(0, os.path.dirname(DIR))
 
 import unittest
+import doctest
 import datetime
 from decimal import Decimal
 import trytond.tests.test_tryton
 from trytond.tests.test_tryton import test_view, test_depends
 from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT
+from trytond.backend.sqlite.database import Database as SQLiteDatabase
 from trytond.transaction import Transaction
 
 
@@ -26,6 +28,8 @@ class AccountTestCase(unittest.TestCase):
     def setUp(self):
         trytond.tests.test_tryton.install_module('account')
         self.account_template = POOL.get('account.account.template')
+        self.tax_code_template = POOL.get('account.tax.code.template')
+        self.tax_template = POOL.get('account.tax.code.template')
         self.account = POOL.get('account.account')
         self.account_create_chart = POOL.get(
             'account.create_chart', type='wizard')
@@ -56,7 +60,36 @@ class AccountTestCase(unittest.TestCase):
             account_template, = self.account_template.search([
                     ('parent', '=', None),
                     ])
-            company, = self.company.search([('name', '=', 'B2CK')])
+            tax_account, = self.account_template.search([
+                    ('name', '=', 'Main Tax'),
+                    ])
+            with Transaction().set_user(0):
+                tax_code = self.tax_code_template()
+                tax_code.name = 'Tax Code'
+                tax_code.account = account_template
+                tax_code.save()
+                base_code = self.tax_code_template()
+                base_code.name = 'Base Code'
+                base_code.account = account_template
+                base_code.save()
+                tax = self.tax_template()
+                tax.name = tax.description = '20% VAT'
+                tax.type = 'percentage'
+                tax.percentage = Decimal(20)
+                tax.account = account_template
+                tax.invoice_account = tax_account
+                tax.credit_note_account = tax_account
+                tax.invoice_base_code = base_code
+                tax.invoice_base_sign = Decimal(1)
+                tax.invoice_tax_code = tax_code
+                tax.invoice_tax_sign = Decimal(1)
+                tax.credit_note_base_code = base_code
+                tax.credit_note_base_sign = Decimal(-1)
+                tax.credit_note_tax_code = tax_code
+                tax.credit_note_tax_sign = Decimal(-1)
+                tax.save()
+
+            company, = self.company.search([('rec_name', '=', 'B2CK')])
             self.user.write([self.user(USER)], {
                     'main_company': company.id,
                     'company': company.id,
@@ -89,19 +122,19 @@ class AccountTestCase(unittest.TestCase):
         with Transaction().start(DB_NAME, USER,
                 context=CONTEXT) as transaction:
             today = datetime.date.today()
-            company, = self.company.search([('name', '=', 'B2CK')])
-            sequence = self.sequence.create({
-                    'name': '%s' % today.year,
-                    'code': 'account.move',
-                    'company': company.id,
-                    })
-            fiscalyear = self.fiscalyear.create({
-                    'name': '%s' % today.year,
-                    'start_date': today.replace(month=1, day=1),
-                    'end_date': today.replace(month=12, day=31),
-                    'company': company.id,
-                    'post_move_sequence': sequence.id,
-                    })
+            company, = self.company.search([('rec_name', '=', 'B2CK')])
+            sequence, = self.sequence.create([{
+                        'name': '%s' % today.year,
+                        'code': 'account.move',
+                        'company': company.id,
+                        }])
+            fiscalyear, = self.fiscalyear.create([{
+                        'name': '%s' % today.year,
+                        'start_date': today.replace(month=1, day=1),
+                        'end_date': today.replace(month=12, day=31),
+                        'company': company.id,
+                        'post_move_sequence': sequence.id,
+                        }])
             self.fiscalyear.create_period([fiscalyear])
             self.assertEqual(len(fiscalyear.periods), 12)
             transaction.cursor.commit()
@@ -133,20 +166,19 @@ class AccountTestCase(unittest.TestCase):
                     ('kind', '=', 'payable'),
                     ])
             # Create some moves
-            values = [
+            vlist = [
                 {
                     'period': period.id,
                     'journal': journal_revenue.id,
                     'date': period.start_date,
                     'lines': [
-                        ('create', {
-                                'account': revenue.id,
-                                'credit': Decimal(100),
-                                }),
-                        ('create', {
-                                'account': receivable.id,
-                                'debit': Decimal(100),
-                                }),
+                        ('create', [{
+                                    'account': revenue.id,
+                                    'credit': Decimal(100),
+                                    }, {
+                                    'account': receivable.id,
+                                    'debit': Decimal(100),
+                                    }]),
                         ],
                     },
                 {
@@ -154,19 +186,17 @@ class AccountTestCase(unittest.TestCase):
                     'journal': journal_expense.id,
                     'date': period.start_date,
                     'lines': [
-                        ('create', {
-                                'account': expense.id,
-                                'debit': Decimal(30),
-                                }),
-                        ('create', {
-                                'account': payable.id,
-                                'credit': Decimal(30),
-                                }),
+                        ('create', [{
+                                    'account': expense.id,
+                                    'debit': Decimal(30),
+                                    }, {
+                                    'account': payable.id,
+                                    'credit': Decimal(30),
+                                    }]),
                         ],
                     },
                 ]
-            for value in values:
-                self.move.create(value)
+            self.move.create(vlist)
 
             # Test debit/credit
             self.assertEqual((revenue.debit, revenue.credit),
@@ -174,11 +204,11 @@ class AccountTestCase(unittest.TestCase):
             self.assertEqual(revenue.balance, Decimal(-100))
 
             # Use next fiscalyear
-            next_sequence = self.sequence.create({
-                    'name': 'Next Year',
-                    'code': 'account.move',
-                    'company': fiscalyear.company.id,
-                    })
+            next_sequence, = self.sequence.create([{
+                        'name': 'Next Year',
+                        'code': 'account.move',
+                        'company': fiscalyear.company.id,
+                        }])
             next_fiscalyear, = self.fiscalyear.copy([fiscalyear],
                 default={
                     'start_date': fiscalyear.end_date + datetime.timedelta(1),
@@ -207,41 +237,39 @@ class AccountTestCase(unittest.TestCase):
             journal_sequence, = self.sequence.search([
                     ('code', '=', 'account.journal'),
                     ])
-            journal_closing = self.journal.create({
-                    'name': 'Closing',
-                    'code': 'CLO',
-                    'type': 'situation',
-                    'sequence': journal_sequence.id,
-                    })
+            journal_closing, = self.journal.create([{
+                        'name': 'Closing',
+                        'code': 'CLO',
+                        'type': 'situation',
+                        'sequence': journal_sequence.id,
+                        }])
             type_equity, = self.account_type.search([
                     ('name', '=', 'Equity'),
                     ])
-            account_pl = self.account.create({
-                    'name': 'P&L',
-                    'type': type_equity.id,
-                    'deferral': True,
-                    'parent': revenue.parent.id,
-                    'kind': 'other',
-                    })
-            self.move.create({
-                    'period': fiscalyear.periods[-1].id,
-                    'journal': journal_closing.id,
-                    'date': fiscalyear.periods[-1].end_date,
-                    'lines': [
-                        ('create', {
-                                'account': revenue.id,
-                                'debit': Decimal(100),
-                                }),
-                        ('create', {
-                                'account': expense.id,
-                                'credit': Decimal(30),
-                                }),
-                        ('create', {
-                                'account': account_pl.id,
-                                'credit': Decimal('70'),
-                                }),
-                        ],
-                    })
+            account_pl, = self.account.create([{
+                        'name': 'P&L',
+                        'type': type_equity.id,
+                        'deferral': True,
+                        'parent': revenue.parent.id,
+                        'kind': 'other',
+                        }])
+            self.move.create([{
+                        'period': fiscalyear.periods[-1].id,
+                        'journal': journal_closing.id,
+                        'date': fiscalyear.periods[-1].end_date,
+                        'lines': [
+                            ('create', [{
+                                        'account': revenue.id,
+                                        'debit': Decimal(100),
+                                        }, {
+                                        'account': expense.id,
+                                        'credit': Decimal(30),
+                                        }, {
+                                        'account': account_pl.id,
+                                        'credit': Decimal('70'),
+                                        }]),
+                            ],
+                        }])
             moves = self.move.search([
                     ('state', '=', 'draft'),
                     ('period.fiscalyear', '=', fiscalyear.id),
@@ -298,6 +326,19 @@ class AccountTestCase(unittest.TestCase):
             transaction.cursor.rollback()
 
 
+def doctest_dropdb(test):
+    '''
+    Remove sqlite memory database
+    '''
+    database = SQLiteDatabase().connect()
+    cursor = database.cursor(autocommit=True)
+    try:
+        database.drop(cursor, ':memory:')
+        cursor.commit()
+    finally:
+        cursor.close()
+
+
 def suite():
     suite = trytond.tests.test_tryton.suite()
     from trytond.modules.company.tests import test_company
@@ -306,6 +347,10 @@ def suite():
             suite.addTest(test)
     suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
         AccountTestCase))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_account_reconciliation.rst',
+            setUp=doctest_dropdb, tearDown=doctest_dropdb, encoding='utf-8',
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     return suite
 
 if __name__ == '__main__':
